@@ -1,6 +1,7 @@
 import copy
 import re
 import typing
+from collections import defaultdict
 
 import uritemplate
 from django.core import exceptions as django_exceptions
@@ -24,13 +25,14 @@ from drf_spectacular.extensions import (
     OpenApiFilterExtension, OpenApiSerializerExtension, OpenApiSerializerFieldExtension,
 )
 from drf_spectacular.plumbing import (
-    ComponentRegistry, ResolvedComponent, UnableToProceedError, append_meta, build_array_type,
-    build_basic_type, build_choice_field, build_examples_list, build_generic_type,
-    build_media_type_object, build_object_type, build_parameter_type, error, follow_field_source,
-    follow_model_field_lookup, force_instance, get_doc, get_type_hints, get_view_model,
-    is_basic_type, is_field, is_list_serializer, is_patched_serializer, is_serializer,
-    is_trivial_string_variation, resolve_django_path_parameter, resolve_regex_path_parameter,
-    resolve_type_hint, safe_ref, warn,
+    ComponentRegistry, ResolvedComponent, UnableToProceedError, append_meta,
+    assert_basic_serializer, build_array_type, build_basic_type, build_choice_field,
+    build_examples_list, build_generic_type, build_media_type_object, build_object_type,
+    build_parameter_type, error, follow_field_source, follow_model_field_lookup, force_instance,
+    get_doc, get_type_hints, get_view_model, is_basic_serializer, is_basic_type, is_field,
+    is_list_serializer, is_patched_serializer, is_serializer, is_trivial_string_variation,
+    resolve_django_path_parameter, resolve_regex_path_parameter, resolve_type_hint, safe_ref,
+    sanitize_specification_extensions, warn,
 )
 from drf_spectacular.settings import spectacular_settings
 from drf_spectacular.types import OpenApiTypes
@@ -85,6 +87,10 @@ class AutoSchema(ViewInspector):
 
         operation['responses'] = self._get_response_bodies()
 
+        extensions = self.get_extensions()
+        if extensions:
+            operation.update(sanitize_specification_extensions(extensions))
+
         return operation
 
     def _is_list_view(self, serializer=None):
@@ -162,8 +168,9 @@ class AutoSchema(ViewInspector):
                         explode=parameter.explode,
                         default=parameter.default,
                         examples=build_examples_list(parameter.examples),
+                        extensions=parameter.extensions,
                     )
-            elif is_serializer(parameter):
+            elif is_basic_serializer(parameter):
                 # explode serializer into separate parameters. defaults to QUERY location
                 mapped = self._map_serializer(parameter, 'request')
                 for property_name, property_schema in mapped['properties'].items():
@@ -214,6 +221,17 @@ class AutoSchema(ViewInspector):
             else:
                 parameters[key] = parameter
 
+        # collect independently specified parameter examples from @extend_schema.
+        # Append to both discovered and manually specified parameters.
+        examples_by_key = defaultdict(list)
+        for example in self.get_examples():
+            if example.parameter_only:
+                examples_by_key[example.parameter_only].append(example)
+        for key, examples in examples_by_key.items():
+            if key in parameters:
+                parameters[key].setdefault('examples', {})
+                parameters[key]['examples'].update(build_examples_list(examples))
+
         if callable(spectacular_settings.SORT_OPERATION_PARAMETERS):
             return sorted(parameters.values(), key=spectacular_settings.SORT_OPERATION_PARAMETERS)
         elif spectacular_settings.SORT_OPERATION_PARAMETERS:
@@ -261,13 +279,20 @@ class AutoSchema(ViewInspector):
             if security_requirements is not None:
                 auths.append(security_requirements)
 
-            component = ResolvedComponent(
-                name=scheme.name,
-                type=ResolvedComponent.SECURITY_SCHEMA,
-                object=authenticator.__class__,
-                schema=scheme.get_security_definition(self)
-            )
-            self.registry.register_on_missing(component)
+            if isinstance(scheme.name, str):
+                names, definitions = [scheme.name], [scheme.get_security_definition(self)]
+            else:
+                names, definitions = scheme.name, scheme.get_security_definition(self)
+
+            for name, definition in zip(names, definitions):
+                self.registry.register_on_missing(
+                    ResolvedComponent(
+                        name=name,
+                        type=ResolvedComponent.SECURITY_SCHEMA,
+                        object=authenticator.__class__,
+                        schema=definition
+                    )
+                )
 
         if spectacular_settings.SECURITY:
             auths.extend(spectacular_settings.SECURITY)
@@ -293,6 +318,9 @@ class AutoSchema(ViewInspector):
         # use first non-parameter path part as tag
         return tokenized_path[:1]
 
+    def get_extensions(self) -> typing.Dict[str, typing.Any]:
+        return {}
+
     def get_operation_id(self):
         """ override this for custom behaviour """
         tokenized_path = self._tokenize_path()
@@ -512,12 +540,7 @@ class AutoSchema(ViewInspector):
 
         # nested serializer with many=True gets automatically replaced with ListSerializer
         if is_list_serializer(field):
-            if is_serializer(field.child):
-                component = self.resolve_serializer(field.child, direction)
-                return append_meta(build_array_type(component.ref), meta) if component else None
-            else:
-                schema = self._map_serializer_field(field.child, direction)
-                return append_meta(build_array_type(schema), meta)
+            return append_meta(self._unwrap_list_serializer(field, direction), meta)
 
         # nested serializer
         if is_serializer(field):
@@ -580,11 +603,12 @@ class AutoSchema(ViewInspector):
         if isinstance(field, serializers.ListField):
             if isinstance(field.child, _UnvalidatedField):
                 return append_meta(build_array_type(build_basic_type(OpenApiTypes.ANY)), meta)
-            elif is_serializer(field.child):
+            elif is_basic_serializer(field.child):
                 component = self.resolve_serializer(field.child, direction)
                 return append_meta(build_array_type(component.ref), meta) if component else None
             else:
                 schema = self._map_serializer_field(field.child, direction)
+                self._insert_field_validators(field.child, schema)
                 # remove automatically attached but redundant title
                 if is_trivial_string_variation(field.field_name, schema.get('title')):
                     schema.pop('title', None)
@@ -629,27 +653,32 @@ class AutoSchema(ViewInspector):
                 content = {**build_basic_type(OpenApiTypes.STR), 'format': 'decimal'}
                 if field.max_whole_digits:
                     content['pattern'] = (
-                        f'^\\d{{0,{field.max_whole_digits}}}'
-                        f'(\\.\\d{{0,{field.decimal_places}}})?$'
+                        fr'^\d{{0,{field.max_whole_digits}}}'
+                        fr'(?:\.\d{{0,{field.decimal_places}}})?$'
                     )
             else:
                 content = build_basic_type(OpenApiTypes.DECIMAL)
                 if field.max_whole_digits:
-                    content['maximum'] = int(field.max_whole_digits * '9') + 1
-                    content['minimum'] = -content['maximum']
-                self._map_min_max(field, content)
+                    value = 10 ** field.max_whole_digits
+                    content.update({
+                        'maximum': value,
+                        'minimum': -value,
+                        'exclusiveMaximum': True,
+                        'exclusiveMinimum': True,
+                    })
+                self._insert_min_max(field, content)
             return append_meta(content, meta)
 
         if isinstance(field, serializers.FloatField):
             content = build_basic_type(OpenApiTypes.FLOAT)
-            self._map_min_max(field, content)
+            self._insert_min_max(field, content)
             return append_meta(content, meta)
 
         if isinstance(field, serializers.IntegerField):
             content = build_basic_type(OpenApiTypes.INT)
-            self._map_min_max(field, content)
-            # 2147483647 is max for int32_size, so we use int64 for format
-            if int(content.get('maximum', 0)) > 2147483647 or int(content.get('minimum', 0)) > 2147483647:
+            self._insert_min_max(field, content)
+            # Use int64 for format if value outside the 32-bit signed integer range [-2,147,483,648 to 2,147,483,647].
+            if not all(-2147483648 <= int(content.get(key, 0)) <= 2147483647 for key in ('maximum', 'minimum')):
                 content['format'] = 'int64'
             return append_meta(content, meta)
 
@@ -675,6 +704,7 @@ class AutoSchema(ViewInspector):
             content = build_basic_type(OpenApiTypes.OBJECT)
             if not isinstance(field.child, _UnvalidatedField):
                 content['additionalProperties'] = self._map_serializer_field(field.child, direction)
+                self._insert_field_validators(field.child, content['additionalProperties'])
             return append_meta(content, meta)
 
         if isinstance(field, serializers.CharField):
@@ -715,11 +745,15 @@ class AutoSchema(ViewInspector):
         warn(f'could not resolve serializer field "{field}". Defaulting to "string"')
         return append_meta(build_basic_type(OpenApiTypes.STR), meta)
 
-    def _map_min_max(self, field, content):
-        if field.max_value:
+    def _insert_min_max(self, field, content):
+        if field.max_value is not None:
             content['maximum'] = field.max_value
-        if field.min_value:
+            if 'exclusiveMaximum' in content:
+                del content['exclusiveMaximum']
+        if field.min_value is not None:
             content['minimum'] = field.min_value
+            if 'exclusiveMinimum' in content:
+                del content['exclusiveMinimum']
 
     def _map_serializer(self, serializer, direction, bypass_extensions=False):
         serializer = force_instance(serializer)
@@ -730,6 +764,10 @@ class AutoSchema(ViewInspector):
         else:
             schema = self._map_basic_serializer(serializer, direction)
 
+        extensions = get_override(serializer, 'extensions', {})
+        if extensions:
+            schema.update(sanitize_specification_extensions(extensions))
+
         return self._postprocess_serializer_schema(schema, serializer, direction)
 
     def _postprocess_serializer_schema(self, schema, serializer, direction):
@@ -788,6 +826,7 @@ class AutoSchema(ViewInspector):
         return meta
 
     def _map_basic_serializer(self, serializer, direction):
+        assert_basic_serializer(serializer)
         serializer = force_instance(serializer)
         required = set()
         properties = {}
@@ -810,7 +849,7 @@ class AutoSchema(ViewInspector):
             if add_to_required:
                 required.add(field.field_name)
 
-            self._map_field_validators(field, schema)
+            self._insert_field_validators(field, schema)
 
             if field.field_name in get_override(serializer, 'deprecate_fields', []):
                 schema['deprecated'] = True
@@ -826,38 +865,62 @@ class AutoSchema(ViewInspector):
             description=get_doc(serializer.__class__),
         )
 
-    def _map_field_validators(self, field, schema):
+    def _insert_field_validators(self, field, schema):
+        schema_type = schema.get('type')
+
+        def update_constraint(schema, key, function, value, *, exclusive=False):
+            if callable(value):
+                value = value()
+            current_value = schema.get(key)
+            if current_value is not None:
+                new_value = function(current_value, value)
+            else:
+                new_value = value
+            schema[key] = new_value
+            if key in ('maximum', 'minimum'):
+                exclusive_key = f'exclusive{key.title()}'
+                if exclusive:
+                    if new_value != current_value:
+                        schema[exclusive_key] = True
+                elif exclusive_key in schema:
+                    del schema[exclusive_key]
+
         for v in field.validators:
-            if isinstance(v, validators.EmailValidator):
-                schema['format'] = 'email'
-            elif isinstance(v, validators.URLValidator):
-                schema['format'] = 'uri'
-            elif isinstance(v, validators.RegexValidator):
-                pattern = v.regex.pattern.encode('ascii', 'backslashreplace').decode()
-                pattern = pattern.replace(r'\x', r'\u00')  # unify escaping
-                pattern = pattern.replace(r'\Z', '$').replace(r'\A', '^')  # ECMA anchors
-                schema['pattern'] = pattern
-            elif isinstance(v, validators.MaxLengthValidator):
-                attr_name = 'maxLength'
-                if isinstance(field, serializers.ListField):
-                    attr_name = 'maxItems'
-                schema[attr_name] = v.limit_value() if callable(v.limit_value) else v.limit_value
-            elif isinstance(v, validators.MinLengthValidator):
-                attr_name = 'minLength'
-                if isinstance(field, serializers.ListField):
-                    attr_name = 'minItems'
-                schema[attr_name] = v.limit_value() if callable(v.limit_value) else v.limit_value
-            elif isinstance(v, validators.MaxValueValidator):
-                schema['maximum'] = v.limit_value() if callable(v.limit_value) else v.limit_value
-            elif isinstance(v, validators.MinValueValidator):
-                schema['minimum'] = v.limit_value() if callable(v.limit_value) else v.limit_value
-            elif isinstance(v, validators.DecimalValidator):
-                if v.max_digits:
-                    digits = v.max_digits
-                    if v.decimal_places is not None and v.decimal_places > 0:
-                        digits -= v.decimal_places
-                    schema['maximum'] = int(digits * '9') + 1
-                    schema['minimum'] = -schema['maximum']
+            if schema_type == 'string':
+                if isinstance(v, validators.EmailValidator):
+                    schema['format'] = 'email'
+                elif isinstance(v, validators.URLValidator):
+                    schema['format'] = 'uri'
+                elif isinstance(v, validators.RegexValidator):
+                    pattern = v.regex.pattern.encode('ascii', 'backslashreplace').decode()
+                    pattern = pattern.replace(r'\x', r'\u00')  # unify escaping
+                    pattern = pattern.replace(r'\Z', '$').replace(r'\A', '^')  # ECMA anchors
+                    schema['pattern'] = pattern
+                elif isinstance(v, validators.MaxLengthValidator):
+                    update_constraint(schema, 'maxLength', min, v.limit_value)
+                elif isinstance(v, validators.MinLengthValidator):
+                    update_constraint(schema, 'minLength', max, v.limit_value)
+                elif isinstance(v, validators.FileExtensionValidator) and v.allowed_extensions:
+                    schema['pattern'] = '(?:%s)$' % '|'.join([re.escape(extn) for extn in v.allowed_extensions])
+            elif schema_type in ('integer', 'number'):
+                if isinstance(v, validators.MaxValueValidator):
+                    update_constraint(schema, 'maximum', min, v.limit_value)
+                elif isinstance(v, validators.MinValueValidator):
+                    update_constraint(schema, 'minimum', max, v.limit_value)
+                elif isinstance(v, validators.DecimalValidator) and v.max_digits:
+                    value = 10 ** (v.max_digits - (v.decimal_places or 0))
+                    update_constraint(schema, 'maximum', min, value, exclusive=True)
+                    update_constraint(schema, 'minimum', max, -value, exclusive=True)
+            elif schema_type == 'array':
+                if isinstance(v, validators.MaxLengthValidator):
+                    update_constraint(schema, 'maxItems', min, v.limit_value)
+                elif isinstance(v, validators.MinLengthValidator):
+                    update_constraint(schema, 'minItems', max, v.limit_value)
+            elif schema_type == 'object':
+                if isinstance(v, validators.MaxLengthValidator):
+                    update_constraint(schema, 'maxProperties', min, v.limit_value)
+                elif isinstance(v, validators.MinLengthValidator):
+                    update_constraint(schema, 'minProperties', max, v.limit_value)
 
     def _map_response_type_hint(self, method):
         hint = get_override(method, 'field') or get_type_hints(method).get('return')
@@ -930,7 +993,10 @@ class AutoSchema(ViewInspector):
         return []
 
     def _get_examples(self, serializer, direction, media_type, status_code=None, extras=None):
-        examples = self.get_examples()
+        """ Handles examples for request/response. purposefully ignores parameter examples """
+
+        # don't let the parameter examples influence the serializer example retrieval
+        examples = [e for e in self.get_examples() if not e.parameter_only]
 
         if not examples:
             if is_list_serializer(serializer):
@@ -995,11 +1061,7 @@ class AutoSchema(ViewInspector):
         serializer = force_instance(serializer)
 
         if is_list_serializer(serializer):
-            if is_serializer(serializer.child):
-                component = self.resolve_serializer(serializer.child, 'request')
-                schema = build_array_type(component.ref)
-            else:
-                schema = build_array_type(self._map_serializer_field(serializer.child, 'request'))
+            schema = self._unwrap_list_serializer(serializer, 'request')
             request_body_required = True
         elif is_serializer(serializer):
             if self.method == 'PATCH':
@@ -1009,7 +1071,7 @@ class AutoSchema(ViewInspector):
                 serializer = copy.copy(serializer)
                 serializer.partial = True
             component = self.resolve_serializer(serializer, 'request')
-            if not component.schema:
+            if not component:
                 # serializer is empty so skip content enumeration
                 return None, False
             schema = component.ref
@@ -1073,6 +1135,18 @@ class AutoSchema(ViewInspector):
             schema['description'] = _('Unspecified response body')
             return {'200': self._get_response_for_code(schema, '200')}
 
+    def _unwrap_list_serializer(self, serializer, direction) -> dict:
+        if is_field(serializer):
+            return self._map_serializer_field(serializer, direction)
+        elif is_basic_serializer(serializer):
+            return self.resolve_serializer(serializer, direction).ref
+        elif is_list_serializer(serializer):
+            return build_array_type(
+                self._unwrap_list_serializer(serializer.child, direction)
+            )
+        else:
+            assert False, 'Serializer is of unknown type.'
+
     def _get_response_for_code(self, serializer, status_code, media_types=None):
         if isinstance(serializer, OpenApiResponse):
             serializer, description, examples = (
@@ -1088,13 +1162,10 @@ class AutoSchema(ViewInspector):
         if not serializer:
             return {**headers, 'description': description or _('No response body')}
         elif is_list_serializer(serializer):
-            if is_serializer(serializer.child):
-                schema = self.resolve_serializer(serializer.child, 'response').ref
-            else:
-                schema = self._map_serializer_field(serializer.child, 'response')
+            schema = self._unwrap_list_serializer(serializer.child, 'response')
         elif is_serializer(serializer):
             component = self.resolve_serializer(serializer, 'response')
-            if not component.schema:
+            if not component:
                 return {**headers, 'description': description or _('No response body')}
             schema = component.ref
         elif is_basic_type(serializer):
@@ -1188,6 +1259,7 @@ class AutoSchema(ViewInspector):
                 explode=parameter.explode,
                 default=parameter.default,
                 examples=build_examples_list(parameter.examples),
+                extensions=parameter.extensions,
             )
             del parameter_type['name']
             del parameter_type['in']
@@ -1224,13 +1296,9 @@ class AutoSchema(ViewInspector):
         return name
 
     def resolve_serializer(self, serializer, direction) -> ResolvedComponent:
-        assert is_serializer(serializer), (
-            f'internal assumption violated because we expected a serializer here and instead '
-            f'got a "{serializer}". This may be the result of another app doing some unexpected '
-            f'magic or an invalid internal call. Feel free to report this as a bug at '
-            f'https://github.com/tfranzel/drf-spectacular/issues '
-        )
+        assert_basic_serializer(serializer)
         serializer = force_instance(serializer)
+
         with add_trace_message(serializer.__class__.__name__):
             component = ResolvedComponent(
                 name=self._get_serializer_name(serializer, direction),
