@@ -13,6 +13,11 @@ from decimal import Decimal
 from enum import Enum
 from typing import DefaultDict, Generic, List, Optional, Type, TypeVar, Union
 
+if sys.version_info >= (3, 8):
+    from typing import Literal, _TypedDictMeta  # type: ignore[attr-defined]
+else:
+    from typing_extensions import Literal, _TypedDictMeta  # type: ignore[attr-defined]
+
 import inflection
 import uritemplate
 from django.apps import apps
@@ -24,14 +29,13 @@ from django.db.models.fields.related_descriptors import (
 from django.db.models.fields.reverse_related import ForeignObjectRel
 from django.db.models.sql.query import Query
 from django.urls.converters import get_converters
-from django.urls.resolvers import (  # type: ignore
+from django.urls.resolvers import (  # type: ignore[attr-defined]
     _PATH_PARAMETER_COMPONENT_RE, RegexPattern, Resolver404, RoutePattern, URLPattern, URLResolver,
     get_resolver,
 )
 from django.utils.functional import Promise, cached_property
 from django.utils.module_loading import import_string
 from django.utils.translation import gettext_lazy as _
-from django.utils.version import PY38
 from rest_framework import exceptions, fields, mixins, serializers, versioning
 from rest_framework.settings import api_settings
 from rest_framework.test import APIRequestFactory
@@ -51,7 +55,7 @@ except ImportError:
     class Choices:  # type: ignore
         pass
 
-if PY38:
+if sys.version_info >= (3, 8):
     CACHED_PROPERTY_FUNCS = (functools.cached_property, cached_property)  # type: ignore
 else:
     CACHED_PROPERTY_FUNCS = (cached_property,)  # type: ignore
@@ -88,6 +92,10 @@ def is_list_serializer(obj) -> bool:
     return isinstance(force_instance(obj), serializers.ListSerializer)
 
 
+def is_basic_serializer(obj) -> bool:
+    return is_serializer(obj) and not is_list_serializer(obj)
+
+
 def is_field(obj):
     # make sure obj is a serializer field and nothing else.
     # guard against serializers because BaseSerializer(Field)
@@ -117,6 +125,15 @@ def is_trivial_string_variation(a: str, b: str):
     return a == b
 
 
+def assert_basic_serializer(serializer):
+    assert is_basic_serializer(serializer), (
+        f'internal assumption violated because we expected a basic serializer here and '
+        f'instead got a "{serializer}". This may be the result of another app doing '
+        f'some unexpected magic or an invalid internal call. Feel free to report this '
+        f'as a bug at https://github.com/tfranzel/drf-spectacular/issues'
+    )
+
+
 @cache
 def get_lib_doc_excludes():
     # do not import on package level due to potential import recursion when loading
@@ -156,8 +173,12 @@ def get_view_model(view, emit_warnings=True):
 
 def get_doc(obj):
     """ get doc string with fallback on obj's base classes (ignoring DRF documentation). """
+    def post_cleanup(doc: str):
+        # also clean up trailing whitespace for each line
+        return '\n'.join(line.rstrip() for line in doc.rstrip().split('\n'))
+
     if not inspect.isclass(obj):
-        return inspect.getdoc(obj) or ''
+        return post_cleanup(inspect.getdoc(obj) or '')
 
     def safe_index(lst, item):
         try:
@@ -170,10 +191,7 @@ def get_doc(obj):
     )
     for cls in obj.__mro__[:lib_barrier]:
         if cls.__doc__:
-            # dedent and cleanup of trailing white space on each line
-            doc = inspect.cleandoc(cls.__doc__)
-            doc = '\n'.join(line.rstrip() for line in doc.rstrip().split('\n'))
-            return doc
+            return post_cleanup(inspect.cleandoc(cls.__doc__))
     return ''
 
 
@@ -283,6 +301,7 @@ def build_parameter_type(
         style=None,
         default=None,
         examples=None,
+        extensions=None,
 ):
     irrelevant_field_meta = ['readOnly', 'writeOnly']
     if location == OpenApiParameter.PATH:
@@ -308,6 +327,8 @@ def build_parameter_type(
         schema['schema']['default'] = default
     if examples:
         schema['examples'] = examples
+    if extensions:
+        schema.update(sanitize_specification_extensions(extensions))
     return schema
 
 
@@ -478,7 +499,7 @@ def _follow_return_type(a_callable):
     if target_type is None:
         return target_type
     origin, args = _get_type_hint_origin(target_type)
-    if origin is typing.Union:
+    if origin is Union:
         type_args = [arg for arg in args if arg is not type(None)]  # noqa: E721
         if len(type_args) > 1:
             warn(
@@ -990,6 +1011,17 @@ def sanitize_result_object(result):
     return result
 
 
+def sanitize_specification_extensions(extensions):
+    # https://spec.openapis.org/oas/v3.0.3#specification-extensions
+    output = {}
+    for key, value in extensions.items():
+        if not re.match(r'^x-', key):
+            warn(f'invalid extension {key!r}. vendor extensions must start with "x-"')
+        else:
+            output[key] = value
+    return output
+
+
 def camelize_operation(path, operation):
     for path_variable in re.findall(r'\{(\w+)\}', path):
         path = path.replace(
@@ -1088,8 +1120,9 @@ def resolve_type_hint(hint):
         return build_array_type(resolve_type_hint(args[0]))
     elif origin is frozenset:
         return build_array_type(resolve_type_hint(args[0]))
-    elif hasattr(typing, 'Literal') and origin is typing.Literal:
-        # python >= 3.8
+    elif origin is Literal:
+        # Literal only works for python >= 3.8 despite typing_extensions, because it
+        # behaves slightly different w.r.t. __origin__
         schema = {'enum': list(args)}
         if all(type(args[0]) is type(choice) for choice in args):
             schema.update(build_basic_type(type(args[0])))
@@ -1100,13 +1133,13 @@ def resolve_type_hint(hint):
         if mixin_base_types:
             schema.update(build_basic_type(mixin_base_types[0]))
         return schema
-    elif hasattr(typing, 'TypedDict') and isinstance(hint, typing._TypedDictMeta):
+    elif isinstance(hint, _TypedDictMeta):
         return build_object_type(
             properties={
                 k: resolve_type_hint(v) for k, v in get_type_hints(hint).items()
             }
         )
-    elif origin is typing.Union:
+    elif origin is Union:
         type_args = [arg for arg in args if arg is not type(None)]  # noqa: E721
         if len(type_args) > 1:
             schema = {'oneOf': [resolve_type_hint(arg) for arg in type_args]}
